@@ -3,8 +3,8 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
-              Vladimír Vondruš <mosra@centrum.cz>
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
+                2020 Vladimír Vondruš <mosra@centrum.cz>
     Copyright © 2015 Jonathan Hale <squareys@googlemail.com>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -55,7 +55,45 @@ namespace Magnum { namespace Trade {
 /**
 @brief DDS image importer plugin
 
-Supports DirectDraw Surface images (`*.dds`) in the following formats:
+Supports DirectDraw Surface images (`*.dds`).
+
+@section Trade-DdsImporter-usage Usage
+
+This plugin depends on the @ref Trade library and is built if
+`WITH_DDSIMPORTER` is enabled when building Magnum Plugins. To use as a dynamic
+plugin, load @cpp "DdsImporter" @ce via @ref Corrade::PluginManager::Manager.
+
+Additionally, if you're using Magnum as a CMake subproject, bundle the
+[magnum-plugins repository](https://github.com/mosra/magnum-plugins) and do the
+following:
+
+@code{.cmake}
+set(WITH_DDSIMPORTER ON CACHE BOOL "" FORCE)
+add_subdirectory(magnum-plugins EXCLUDE_FROM_ALL)
+
+# So the dynamically loaded plugin gets built implicitly
+add_dependencies(your-app MagnumPlugins::DdsImporter)
+@endcode
+
+To use as a static plugin or as a dependency of another plugin with CMake, put
+[FindMagnumPlugins.cmake](https://github.com/mosra/magnum-plugins/blob/master/modules/FindMagnumPlugins.cmake)
+into your `modules/` directory, request the `DdsImporter` component of the
+`MagnumPlugins` package in CMake and link to the `MagnumPlugins::DdsImporter`
+target:
+
+@code{.cmake}
+find_package(MagnumPlugins REQUIRED DdsImporter)
+
+# ...
+target_link_libraries(your-app PRIVATE MagnumPlugins::DdsImporter)
+@endcode
+
+See @ref building-plugins, @ref cmake-plugins, @ref plugins and
+@ref file-formats for more information.
+
+@section Trade-DdsImporter-behavior Behavior and limitations
+
+Imports images in the following formats:
 
 -   DDS uncompressed RGB, RGBA, BGR, BGRA, grayscale as
     @ref PixelFormat::RGB8Unorm, @ref PixelFormat::RGBA8Unorm or
@@ -101,18 +139,10 @@ Supports DirectDraw Surface images (`*.dds`) in the following formats:
     -   `R32_FLOAT`, `R32G32_FLOAT`, `R32G32B32_FLOAT`, `R32G32B32A32_FLOAT` as
         @ref PixelFormat::R32F and its two-/three-/four-component equivalents
 
-This plugin depends on the @ref Trade library and is built if
-`WITH_DDSIMPORTER` is enabled when building Magnum Plugins. To use as a dynamic
-plugin, you need to load the @cpp "DdsImporter" @ce plugin from
-`MAGNUM_PLUGINS_IMPORTER_DIR`. To use as a static plugin or as a dependency of
-another plugin with CMake, you need to request the `DdsImporter` component of
-the `MagnumPlugins` package in CMake and link to the
-`MagnumPlugins::DdsImporter` target. See @ref building-plugins,
-@ref cmake-plugins and @ref plugins for more information.
+The importer recognizes @ref ImporterFlag::Verbose, printing additional info
+when the flag is enabled.
 
-Note: Mipmaps are currently imported under separate image data ids. You may
-access them via @ref image2D(UnsignedInt)/@ref image3D(UnsignedInt) which will
-return the n-th mip, a bigger n indicating a smaller mip.
+BC6h, BC7 and other compressed formats are currently not imported correctly.
 */
 class MAGNUM_DDSIMPORTER_EXPORT DdsImporter: public AbstractImporter {
     public:
@@ -125,21 +155,22 @@ class MAGNUM_DDSIMPORTER_EXPORT DdsImporter: public AbstractImporter {
         ~DdsImporter();
 
     private:
-        MAGNUM_DDSIMPORTER_LOCAL Features doFeatures() const override;
+        MAGNUM_DDSIMPORTER_LOCAL ImporterFeatures doFeatures() const override;
         MAGNUM_DDSIMPORTER_LOCAL bool doIsOpened() const override;
         MAGNUM_DDSIMPORTER_LOCAL void doClose() override;
         MAGNUM_DDSIMPORTER_LOCAL void doOpenData(Containers::ArrayView<const char> data) override;
 
         MAGNUM_DDSIMPORTER_LOCAL UnsignedInt doImage2DCount() const override;
-        MAGNUM_DDSIMPORTER_LOCAL Containers::Optional<ImageData2D> doImage2D(UnsignedInt id) override;
+        MAGNUM_DDSIMPORTER_LOCAL UnsignedInt doImage2DLevelCount(UnsignedInt id) override;
+        MAGNUM_DDSIMPORTER_LOCAL Containers::Optional<ImageData2D> doImage2D(UnsignedInt id, UnsignedInt level) override;
 
         MAGNUM_DDSIMPORTER_LOCAL UnsignedInt doImage3DCount() const override;
-        MAGNUM_DDSIMPORTER_LOCAL Containers::Optional<ImageData3D> doImage3D(UnsignedInt id) override;
+        MAGNUM_DDSIMPORTER_LOCAL UnsignedInt doImage3DLevelCount(UnsignedInt id) override;
+        MAGNUM_DDSIMPORTER_LOCAL Containers::Optional<ImageData3D> doImage3D(UnsignedInt id, UnsignedInt level) override;
 
     private:
         struct File;
-
-        std::unique_ptr<File> _f;
+        Containers::Pointer<File> _f;
 };
 
 }}
